import numpy as np
import matplotlib.pyplot as plt
import matplotlib.colors as colors
import pyopenms as oms
from matplotlib import image as mpimg
from torch.onnx.symbolic_opset12 import argmax

from pyRawMSDataReader.pyRawMSDataReader.WiffFileReader_py import WiffFileReader


def build_image_ms1_wiff(path, bin_mz):
    #load raw data
    rawFile = WiffFileReader(path)
    max_cycle=0
    total_scan_number = rawFile.GetLastSpectrumNumber()
    for scanNumber in range (total_scan_number):
        if rawFile.GetMSOrderForScanNum(scanNumber) == 1 :
            ms1_start_mz = rawFile.source.ScanInfos[scanNumber].LowMz
            ms1_end_mz = rawFile.source.ScanInfos[scanNumber].HighMz
            max_cycle+=1

    print('start', ms1_start_mz, 'end', ms1_end_mz)
    total_ms1_mz = ms1_end_mz - ms1_start_mz

    n_bin_ms1 = int(total_ms1_mz // bin_mz)
    size_bin_ms1 = total_ms1_mz / n_bin_ms1
    im = np.zeros([max_cycle, n_bin_ms1])

    cycle = 0
    for scanNumber in range(total_scan_number):
        if rawFile.GetMSOrderForScanNum(scanNumber) == 1:
            masses, intensities = rawFile.GetCentroidMassListFromScanNum(scanNumber)
            line = np.zeros(n_bin_ms1)
            if len(masses) > 0:
                for k in range(len(masses)):
                    line[int((masses[k] - ms1_start_mz) // size_bin_ms1)] += intensities[k]
            im[cycle, :] = line
            cycle += 1

    return im

def build_image_ms1_wiff_charge_filtered(path, bin_mz=1, tolerance=0.01, noise=10000):
    #load raw data
    rawFile = WiffFileReader(path)
    max_cycle=0
    total_scan_number = rawFile.GetLastSpectrumNumber()
    for scanNumber in range (total_scan_number):
        if rawFile.GetMSOrderForScanNum(scanNumber) == 1 :
            ms1_start_mz = rawFile.source.ScanInfos[scanNumber].LowMz
            ms1_end_mz = rawFile.source.ScanInfos[scanNumber].HighMz
            max_cycle+=1

    print('start', ms1_start_mz, 'end', ms1_end_mz)
    total_ms1_mz = ms1_end_mz - ms1_start_mz

    n_bin_ms1 = int(total_ms1_mz // bin_mz)
    size_bin_ms1 = total_ms1_mz / n_bin_ms1
    im = np.zeros([max_cycle, n_bin_ms1])

    cycle = 0
    for scanNumber in range(total_scan_number):
        if rawFile.GetMSOrderForScanNum(scanNumber) == 1:
            masses, intensities = rawFile.GetCentroidMassListFromScanNum(scanNumber)
            is_bicharged = [False for i in range(len(masses))]
            is_tricharged = [False for i in range(len(masses))]
            for i in range(len(masses)):
                m = masses[i]
                found_b = True
                found_t = True
                b=1
                t=1
                if not is_bicharged[i]:
                    dec = 1
                    while found_b :
                        found_b=False
                        while (i + dec) < len(masses) and masses[i + dec] - m < (b / 2 + tolerance):
                            if masses[i + dec] - m > (b / 2 - tolerance):
                                is_bicharged[i] = True
                                is_bicharged[i + dec] = True
                                found_b = True
                            dec += 1
                        b += 1

                if not is_tricharged[i]:
                    dec = 1
                    while found_t :
                        found_t=False
                        while (i + dec) < len(masses) and masses[i + dec] - m < (1 / 3 + tolerance):
                            if masses[i + dec] - m > (t / 3 - tolerance):
                                is_tricharged[i] = True
                                is_tricharged[i + dec] = True
                                found_t = True
                            dec += 1
                        t += 1
            line = np.zeros(n_bin_ms1)
            if len(masses) > 0:
                for k in range(len(masses)):
                    if (is_bicharged[k] or is_tricharged[k]) and intensities[k]>=noise:
                        line[int((masses[k] - ms1_start_mz) // size_bin_ms1)] += intensities[k]
            im[cycle, :] = line
            cycle += 1
    rawFile.Close()
    return im

#TODO calculer les apex au fil de l'eau, trop long sinon
def build_image_ms1_wiff_charge_filtered_apex_only(path, bin_mz=1, tolerance=0.01, noise=10000):
    #load raw data
    rawFile = WiffFileReader(path)
    max_cycle=0
    total_scan_number = rawFile.GetLastSpectrumNumber()
    for scanNumber in range (total_scan_number):
        if rawFile.GetMSOrderForScanNum(scanNumber) == 1 :
            ms1_start_mz = rawFile.source.ScanInfos[scanNumber].LowMz
            ms1_end_mz = rawFile.source.ScanInfos[scanNumber].HighMz
            max_cycle+=1

    print('start', ms1_start_mz, 'end', ms1_end_mz)
    total_ms1_mz = ms1_end_mz - ms1_start_mz

    n_bin_ms1 = int(total_ms1_mz // bin_mz)
    size_bin_ms1 = total_ms1_mz / n_bin_ms1


    mass_list_ms1=[]
    intensity_list_ms1 = []
    for scanNumber in range(total_scan_number):
        if rawFile.GetMSOrderForScanNum(scanNumber) == 1:
            masses, intensities = rawFile.GetCentroidMassListFromScanNum(scanNumber)
            mass_list_ms1.append(masses)
            intensity_list_ms1.append(intensities)
    rawFile.Close()

    cluster_list=[]
    cluster_ref_2 = []
    cluster_ref_3 = []

    for cycle in range(len(mass_list_ms1)):
        for i in range(len(mass_list_ms1[cycle])):
            is_in_cluster = False
            mass, intensity = mass_list_ms1[cycle][i], intensity_list_ms1[cycle][i]
            if intensity>=noise:
                #check id peak belongs to a previous cluster
                for cycle_ref,mass_ref,cluster_id in cluster_ref_2 :
                    if cycle_ref==cycle:
                        if mass_ref - tolerance < mass_list_ms1[cycle][i] < mass_ref + tolerance :
                            if mass - cluster_list[cluster_id][0][1] > 1/(3 - 2 * tolerance) :
                                cluster_list[cluster_id] = [max([cluster_list[cluster_id][0],(cycle,mass,intensity)],key=lambda x : x[2]),cluster_list[cluster_id][1]+ 1]
                            else :
                                 cluster_list[cluster_id] = [max([cluster_list[cluster_id][0], (cycle, mass, intensity)], key=lambda x: x[2]),cluster_list[cluster_id][1] + 1]
                            if not (cycle + 1,mass_ref,  cluster_id) in cluster_ref_2:
                                cluster_ref_2.append((cycle + 1,mass_ref,  cluster_id))
                            is_in_cluster = True
                            if not (cycle,mass_ref+1/2,cluster_id) in cluster_ref_2 :
                                cluster_ref_2.append((cycle,mass_ref+1/2,cluster_id))

                for cycle_ref,mass_ref,cluster_id in cluster_ref_3 :
                    if cycle_ref==cycle:
                        if mass_ref - tolerance < mass_list_ms1[cycle][i] < mass_ref + tolerance :
                            cluster_list[cluster_id].append((cycle,mass,intensity))
                            if not ( cycle + 1,mass_ref, cluster_id) in cluster_ref_3:
                                cluster_ref_3.append(( cycle + 1,mass_ref, cluster_id))
                            is_in_cluster = True
                            if not (cycle,mass_ref+1/3,cluster_id) in cluster_ref_3 :
                                cluster_ref_3.append((cycle,mass_ref+1/3,cluster_id))
                #else create a new cluster based on this peak
                if not is_in_cluster :
                    cluster_ref_2.append((cycle,mass+1/2,len(cluster_list)))
                    cluster_ref_2.append((cycle+1, mass, len(cluster_list)))
                    cluster_list.append([(0,0,0),0]) #(cycle mass intensity),number of heavy isotop
                    cluster_ref_3.append((cycle, mass + 1 / 3, len(cluster_list)))
                    cluster_ref_3.append((cycle + 1, mass, len(cluster_list)))
                    cluster_list.append([(0,0,0),0]) #(cycle mass intensity),number of heavy isotop

        #remove old cluster ref to improve perf (optional)
        for cycle_ref,mass_ref,cluster_id in cluster_ref_2 :
            if cycle_ref == cycle :
                cluster_ref_2= [x for x in cluster_ref_2 if x != (cycle_ref,mass_ref,cluster_id)]
        for cycle_ref,mass_ref, cluster_id in cluster_ref_3 :
            if cycle_ref == cycle :
                cluster_ref_3 = [x for x in cluster_ref_3 if x != (cycle_ref,mass_ref,cluster_id)]

    apex_list = []
    for cluster in cluster_list : # cluster = (cycle,mass,intensity)
        if cluster[1]>=1: #at least a heavy isotop detected (valid istope group)
            apex_list.append(cluster[0])

    #create image based on detected valid apexes
    im = np.zeros([max_cycle, n_bin_ms1])
    for cycle, mass, _ in apex_list:
        im[int(cycle), int((mass - ms1_start_mz) // size_bin_ms1)] = 1
    return im


def build_image_ms1(path, bin_mz):
    #load raw data
    e = oms.MSExperiment()
    oms.MzMLFile().load(path, e)
    e.updateRanges()
    id_spectra = e.getSpectra()[-1].getNativeID()

    dico = dict(s.split('=', 1) for s in id_spectra.split())
    max_cycle = int(dico['cycle'])
    list_cycle = [[] for _ in range(max_cycle)]
    #get ms window size from first ms1 spectra (similar for all ms1 spectra)
    ms1_start_mz,ms1_end_mz=None,None
    for s in e:
        if s.getMSLevel() == 1:
            ms1_start_mz = s.getInstrumentSettings().getScanWindows()[0].begin
            ms1_end_mz = s.getInstrumentSettings().getScanWindows()[0].end
            break

    total_ms1_mz = ms1_end_mz - ms1_start_mz
    print('start',ms1_start_mz,'end',ms1_end_mz)
    n_bin_ms1 = int(total_ms1_mz//bin_mz)
    size_bin_ms1 = total_ms1_mz / n_bin_ms1
    #organise sepctra by their MSlevel (only MS1 are kept)
    for spec in e:  # data structure
        id_spectra = spec.getNativeID()
        dico = dict(s.split('=', 1) for s in id_spectra.split())
        if spec.getMSLevel() == 1:
            list_cycle[int(dico['cycle']) - 1].insert(0, spec)

    im = np.zeros([max_cycle, n_bin_ms1])
    for c in range(max_cycle):  # Build image line by line
        line = np.zeros(n_bin_ms1)
        if len(list_cycle[c]) > 0:
            for k in range(len(list_cycle[c])):
                ms1 = list_cycle[c][k]
                intensity = ms1.get_peaks()[1]
                mz = ms1.get_peaks()[0]
                for i in range(ms1.size()):
                    line[int((mz[i] - ms1_start_mz) // size_bin_ms1)] += intensity[i]

        im[c, :] = line

    return im


if __name__ == '__main__':
    path = '../data/raw_data/SERMAR-10-AER-d200.wiff'
    mat = build_image_ms1_wiff_charge_filtered_apex_only(path)
    mpimg.imsave('test.png', mat)